#if !defined(LKT_CMD_H__)
#define LKT_CMD_H__

#if defined(__cplusplus)
extern "C" {
#endif

#include <lektor/common.h>

#define CMD_OPT_NULL               \
    {                              \
        .name = NULL, .call = NULL \
    }
#define CMD_OPT_DEFAULT(func)      \
    {                              \
        .name = NULL, .call = func \
    }
#define CMD_ENV_NULL                                  \
    {                                                 \
        .name = NULL, .value = NULL, .handler = NULL, \
    }
#define CMD_ENV(n, hndl)           \
    {                              \
        .name = n, .handler = hndl \
    }
#define CMD_ENV_DEFAULT(n, v, hndl)            \
    {                                          \
        .name = n, .value = v, .handler = hndl \
    }

/* The arguments passed to a command */
struct cmd_args {
    int argc;          /* The number of arguments passed.  */
    const char **argv; /* An array of argument passed.     */
};

#define CMD_ARGS_FROM(argc, argv)      \
    {                                  \
        .argc = (argc), .argv = (argv) \
    }
#define CMD_ARGS_NEXT_FROM(argc, argv)                       \
    {                                                        \
        .argc = argc - 1, .argv = (const char **)&(argv[1]), \
    }
#define CMD_ARGS_NEXT(args)                               \
    {                                                     \
        .argc = (args).argc - 1, .argv = (args).argv + 1, \
    }

/* Callback to call on command, takes the arguments */
typedef void (*cmd_callback)(struct cmd_args *);

/* Callback for the 'apply_env' function. */
typedef void (*cmd_env_callback)(const char *name, const char *value, void *user);

/* The 'option' command struct, this is how a new command is registered for the
 * cmd_parse command. The list must end with the CMD_OPT_NULL or
 * CMD_OPT_DEFAULT macros (set the NULL at the end of a list with no
 * length...). The command names MUST NOT include the = or : characters! They
 * also must be distinguishable, i.e. one command name must not be simply
 * included in another name:
 * - play, playlist => bad, because play is included in playlist
 * - status, start  => good, no inclusion between the names */
struct cmd_opt {
    const char *name;
    cmd_callback call;
};

/* Options or env of the command. Don't depend on the POSIX env variables for
 * now and do it ourself. */
struct cmd_env {
    const char *name;
    const char *value;
    cmd_env_callback handler;
};

/* Parse the 'env' of the command, i.e. the options before the command, like:
 * ./a.out opt1=foo opt2=bar sub-command com1
 * The cmd_apply function is here to do the handle stuff for the options to be
 * registered in a user specific way (callback + user pointer). */
void cmd_parse_env(struct cmd_env *env, int *argc, const char ***argv);
void cmd_apply_env(struct cmd_env *env, void *user);

/* Parse the command line with the list of options. No parameter may be NULL.
 * Does not return. */
EXIT_FUNCTION cmd_parse(struct cmd_opt *opts, int argc, const char **argv);

/* Must be setted for the lkt_cmd_parse function to display the correct help
 * in case of errors. */
void cmd_set_executable_name(const char *);
const char *cmd_get_executable_name(void);

/* Prints the help if possible and exit. */
EXIT_FUNCTION print_help(void);

#if defined(__cplusplus)
}
#endif
#endif /* LKT_CMD_H__ */
