#if !defined(LKT_COMMANDS_H__)
#define LKT_COMMANDS_H__

#if defined(__cplusplus)
extern "C" {
#endif

#include <lektor/common.h>
#include <lektor/net.h>
#include <lektor/database.h>

// clang-format off

/* The list of commands implemented by lektor.
 *
 * Refer to the following link for the description of each command:
 * <https://www.musicpd.org/doc/html/protocol.html> */

/* Querying lektor's status */
bool command_currentsong(struct lkt_state *srv, size_t c, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_status     (struct lkt_state *srv, size_t c, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_stats      (struct lkt_state *srv, size_t c, char *[LKT_MESSAGE_ARGS_MAX]);

/* Controlling playback */
bool command_next    (struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_pause   (struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_previous(struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_play    (struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_playid  (struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_stop    (struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);

bool command_seek    (struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_seekid  (struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_seekcur (struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);

bool command_set_pos(struct lkt_state *srv, int index); /* FIXME: Not used in net/listen.c */

/* The queue */
bool command_add    (struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX], int priority);
bool command_addid  (struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_del    (struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_delid  (struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_clear  (struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_crop   (struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_move   (struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_shuffle(struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_dump   (struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_swap   (struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_swapid (struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_flat   (struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);

PRIVATE_FUNCTION bool
command_add_1(struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX])
{
    return command_add(srv, args, 1);
}

PRIVATE_FUNCTION bool
command_add_5(struct lkt_state *srv, char *args[LKT_MESSAGE_ARGS_MAX])
{
    return command_add(srv, args, 5);
}

bool command_queue_list  (struct lkt_state *srv, size_t c, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_queue_listid(struct lkt_state *srv, size_t c, char *[LKT_MESSAGE_ARGS_MAX]);

/* The playlists */
bool command_plt_create(struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_plt_remove(struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_plt_clear (struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_plt_rename(struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_plt_export(struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_plt_import(struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_plt_add   (struct lkt_state *srv, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_plt_list  (struct lkt_state *srv, size_t, char *[LKT_MESSAGE_ARGS_MAX], int cont);
bool command_plt_ctx   (struct lkt_state *srv, size_t, char *[LKT_MESSAGE_ARGS_MAX], int cont);

/* The help */
bool command_help(struct lkt_state *srv, size_t c, char *[LKT_MESSAGE_ARGS_MAX]);

/* Respond to the idle command */
bool command_idle(struct lkt_state *srv, size_t c, char *[LKT_MESSAGE_ARGS_MAX]);

/* Cancel the call to the `idle` command. */
bool command_noidle(struct lkt_state *srv, size_t c, char *[LKT_MESSAGE_ARGS_MAX]);

/* Find and send karas in the db that match the search expression */
bool command_find(struct lkt_state *srv, size_t c, char *cmd_args[LKT_MESSAGE_ARGS_MAX],
                  long continuation, database_search_init_func init);

/* Set options for the lektor such as `random`, `single`, `repeat`, etc. Comes
 * with helpers functions for callback support (trie structure). */
typedef enum {
    LKT_PLAYBACK_OPTION_NONE,
    LKT_PLAYBACK_OPTION_RANDOM,
    LKT_PLAYBACK_OPTION_SINGLE,
    LKT_PLAYBACK_OPTION_REPEAT,
    LKT_PLAYBACK_OPTION_CONSUME,
    LKT_PLAYBACK_OPTION_VOLUME,
} LKT_PLAYBACK_OPTION;

bool command_set_playback_option(struct lkt_state *, size_t, LKT_PLAYBACK_OPTION, char *[LKT_MESSAGE_ARGS_MAX]);

PRIVATE_FUNCTION bool
command_set_playback_option_random(struct lkt_state *srv, size_t c, char *args[LKT_MESSAGE_ARGS_MAX])
{
    return command_set_playback_option(srv, c, LKT_PLAYBACK_OPTION_RANDOM, args);
}

PRIVATE_FUNCTION bool
command_set_playback_option_repeat(struct lkt_state *srv, size_t c, char *args[LKT_MESSAGE_ARGS_MAX])
{
    return command_set_playback_option(srv, c, LKT_PLAYBACK_OPTION_REPEAT, args);
}

PRIVATE_FUNCTION bool
command_set_playback_option_volume(struct lkt_state *srv, size_t c, char *args[LKT_MESSAGE_ARGS_MAX])
{
    return command_set_playback_option(srv, c, LKT_PLAYBACK_OPTION_VOLUME, args);
}

PRIVATE_FUNCTION bool
command_set_playback_option_single(struct lkt_state *srv, size_t c, char *args[LKT_MESSAGE_ARGS_MAX])
{
    return command_set_playback_option(srv, c, LKT_PLAYBACK_OPTION_SINGLE, args);
}

PRIVATE_FUNCTION bool
command_set_playback_option_consume(struct lkt_state *srv, size_t c, char *args[LKT_MESSAGE_ARGS_MAX])
{
    return command_set_playback_option(srv, c, LKT_PLAYBACK_OPTION_CONSUME, args);
}

/* Authentificate users */
bool command_password(struct lkt_state *srv, size_t c, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_user_add(struct lkt_state *srv, size_t c, char *[LKT_MESSAGE_ARGS_MAX]);

/* Program management control */
bool command_restart(struct lkt_state *srv, size_t c, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_kill   (struct lkt_state *srv, size_t c, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_import (struct lkt_state *srv, size_t c, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_update (struct lkt_state *srv, size_t c, char *[LKT_MESSAGE_ARGS_MAX]);
bool command_rescan (struct lkt_state *srv, size_t c, char *[LKT_MESSAGE_ARGS_MAX], int forced);
bool command_config (struct lkt_state *srv, size_t c, char *[LKT_MESSAGE_ARGS_MAX]);

PRIVATE_FUNCTION bool
command_rescan_true(struct lkt_state *srv, size_t c, char *argv[LKT_MESSAGE_ARGS_MAX])
{
    /* Force the rescan, don't check timestamps */
    return command_rescan(srv, c, argv, true);
}

PRIVATE_FUNCTION bool
command_rescan_false(struct lkt_state *srv, size_t c, char *argv[LKT_MESSAGE_ARGS_MAX])
{
    /* Don't force the rescan, check timestamps */
    return command_rescan(srv, c, argv, false);
}

/* Sticker management. Handle all the sticker commands, the sticker command in
 * MPD is has a lot of subcommands... */
bool command_sticker_handle(struct lkt_state *srv, size_t c, char *[LKT_MESSAGE_ARGS_MAX]);

// clang-format on

#if defined(__cplusplus)
}
#endif
#endif /* LKT_COMMANDS_H__ */
