#if !defined(__LKT_DATABASE_MACRO_H__)
#define __LKT_DATABASE_MACRO_H__

#if !defined(NDEBUG) /* DEBUG */
#define ___LOCAL_DEBUG(...) LOG_DEBUG("DB", __VA_ARGS__)
#else /* NOT DEBUG */
#define ___LOCAL_DEBUG(...)
#endif

#define SQLITE_PREPARE(db, stmt, SQL, goto_label)                                             \
    {                                                                                         \
        if (sqlite3_prepare_v2((sqlite3 *)db, SQL, -1, &(stmt), 0) != SQLITE_OK) {            \
            ___LOCAL_DEBUG("Failed to prepare statement: %s", sqlite3_errmsg((sqlite3 *)db)); \
            goto goto_label;                                                                  \
        }                                                                                     \
    }

#define SQLITE_EXEC(db, SQL, goto_label)                                                   \
    {                                                                                      \
        if (sqlite3_exec((sqlite3 *)db, SQL, NULL, NULL, NULL) != SQLITE_OK) {             \
            ___LOCAL_DEBUG("Failed to exec statement: %s", sqlite3_errmsg((sqlite3 *)db)); \
            goto goto_label;                                                               \
        }                                                                                  \
    }

#define SQLITE_BIND_TEXT(db, stmt, pos, text, error)                                        \
    {                                                                                       \
        if (sqlite3_bind_text(stmt, pos, text, -1, 0) != SQLITE_OK) {                       \
            ___LOCAL_DEBUG("Failed to bind text %s at pos %d: %s", (const char *)text, pos, \
                           sqlite3_errmsg((sqlite3 *)db));                                  \
            goto error;                                                                     \
        }                                                                                   \
    }

#define SQLITE_BIND_INT(db, stmt, pos, integer, error)                          \
    {                                                                           \
        if (sqlite3_bind_int(stmt, pos, integer) != SQLITE_OK) {                \
            ___LOCAL_DEBUG("Failed to bind int %d at pos %d: %s", integer, pos, \
                           sqlite3_errmsg((sqlite3 *)db));                      \
            goto error;                                                         \
        }                                                                       \
    }

#define SQLITE_STEP(db, stmt, code, error)                                       \
    {                                                                            \
        if (sqlite3_step(stmt) != code) {                                        \
            ___LOCAL_DEBUG("Failed to step: %s", sqlite3_errmsg((sqlite3 *)db)); \
            goto error;                                                          \
        }                                                                        \
    }

#define SQLITE_STEP_OK(db, stmt, error)                                               \
    {                                                                                 \
        int ___sqlite_step_ok_ret_code = sqlite3_step(stmt);                          \
        if (___sqlite_step_ok_ret_code != SQLITE_OK &&                                \
            ___sqlite_step_ok_ret_code != SQLITE_DONE) {                              \
            ___LOCAL_DEBUG("Failed to exec step: %s", sqlite3_errmsg((sqlite3 *)db)); \
            goto error;                                                               \
        }                                                                             \
    }

#define SQLITE_STEP_ROW(db, stmt, error)  SQLITE_STEP(db, stmt, SQLITE_ROW, error)
#define SQLITE_STEP_DONE(db, stmt, error) SQLITE_STEP(db, stmt, SQLITE_DONE, error)

#define SQLITE_DO_ROLLBACK(db)                \
    LOG_WARN("DB-DEBUG", "DO ROLLBACK \\o/"); \
    sqlite3_exec((sqlite3 *)db, "ROLLBACK TRANSACTION;\n", NULL, NULL, NULL);

#define sqlite3_column_chars (const char *)sqlite3_column_text

#define ___INVALID_CHARS_DBPATH    ":?!'\""
#define ___is_sql_str_invalid(str) strcspn(str, ___INVALID_CHARS_DBPATH) != strlen(str)

#endif /* __LKT_DATABASE_MACRO_H__ */
