#if !defined(LKT_URI_H__)
#define LKT_URI_H__

#if defined(__cplusplus)
extern "C" {
#endif

#include <stddef.h>

typedef enum {
    URI_NULL,
    URI_ID,
    URI_PLAYLIST,
    URI_TYPE,
    URI_AUTHOR,
    URI_CATEGORY,
    URI_LANGUAGE,
    URI_QUERY,
} LKT_URI_TYPE;

struct lkt_uri {
    LKT_URI_TYPE type;
    const char *column_name;
    union {
        void *value;
        size_t id;
    };
    bool _allocated;
    bool is_int;
};

/* Create and delete URIs */
bool lkt_uri_from_single(struct lkt_uri *ret, char *const str);
bool lkt_uri_from_list(struct lkt_uri *ret, char **list);
void lkt_uri_free(struct lkt_uri *ret);

// clang-format off
#define lkt_uri_from(ret, from) _Generic((from), char * : lkt_uri_from_single, \
                                                 char ** : lkt_uri_from_list)(ret, from)
// clang-format on

/* Make an URL to download from kurisu.
   base_url is the prefix for the api, e.g. https://kurisu.iiens.net/api_kara.php */
int lkt_uri_to_url(struct lkt_uri *uri, const char *base_url, char *ret, size_t len);

#if defined(__cplusplus)
}
#endif
#endif /* LKT_URI_H__ */
