#define _POSIX_C_SOURCE 200809L

#include <lektor/common.h>

#if defined(__linux__)
#include <sys/prctl.h>
#include <sys/types.h>
#endif

#if defined(__FreeBSD__)
#include <sys/thr.h>
#endif

int
lkt_read_self_exe(char *path, size_t len)
{
    return !((readlink(SELF_EXECUTABLE_LINUX, path, len - 1) > 0) ||
             (readlink(SELF_EXECUTABLE_FREEBSD, path, len - 1) > 0) ||
             (readlink(SELF_EXECUTABLE_SOLARIS, path, len - 1) > 0));
}

void
lkt_thread_set_name(const char *name)
{
    const size_t len = strlen(name);
    if (len >= 16) {
        LOG_FATAL("The name should at least be a 15 character string! Here the length is %zu", len);
    }

#if defined(__linux__)
    pthread_t tid = pthread_self();
    LOG_INFO("OS", "Set name to '%s' for thread %ld", name, tid);
    prctl(PR_SET_NAME, name);
#elif defined(__FreeBSD__)
    long id;
    assert(0 == thr_self(&id));
    LOG_INFO("OS", "Set name to '%s' for thread %ld", name, id);
    assert(0 == thr_set_name(id, name));
#else
#pragma message(TODO "Add support for thread set name for the current OS")
    LOG_ERROR("OS", "Unsuported operation 'thread_set_name' for OS");
#endif
}

/* Endian stuff */

float
be_float_t(const uint8_t bytes[])
{
    union {
        uint8_t _data[4];
        uint32_t _uint;
        float _float;
    } ret;

#if defined(LKT_ENDIANES_LITTLE) && (LKT_ENDIANES_LITTLE == 1)
    for (size_t i = 0; i < 4; ++i)
        ret._data[i] = bytes[3 - i];
#else /* BIG ENDIAN */
    for (size_t i = 0; i < 4; ++i)
        ret._data[i] = bytes[i];
#endif

    return ret._float;
}

double
be_double_t(const uint8_t bytes[])
{
    union {
        uint8_t _data[8];
        uint64_t _uint;
        double _float;
    } ret;

#if defined(LKT_ENDIANES_LITTLE) && (LKT_ENDIANES_LITTLE == 1)
    for (size_t i = 0; i < 8; ++i)
        ret._data[i] = bytes[7 - i];
#else /* BIG ENDIAN */
    for (size_t i = 0; i < 8; ++i)
        ret._data[i] = bytes[i];
#endif

    return ret._float;
}

uint64_t
be_uint64_t(const uint8_t bytes[], size_t n)
{
    uint64_t res = 0;
    for (size_t i = 0; i < n; i++) {
#if defined(LKT_ENDIANES_LITTLE) && (LKT_ENDIANES_LITTLE == 1)
        res = (res << 8u) | bytes[i];
#else /* BIG_ENDIAN */
        res          = res | (bytes[i] << 8u);
#endif
    }
    return res;
}

uint32_t
be_uint32_t(const uint8_t bytes[], size_t n)
{
    uint32_t res = 0;
    for (size_t i = 0; i < n; i++) {
#if defined(LKT_ENDIANES_LITTLE) && (LKT_ENDIANES_LITTLE == 1)
        res = (res << 8u) | bytes[i];
#else /* BIG_ENDIAN */
        res          = res | (bytes[i] << 8u);
#endif
    }
    return res;
}
