#define _POSIX_C_SOURCE 200809L

/* Try to print the backtrace when a segfault occures if possible */

#include <lektor/common.h>
#include <execinfo.h>
#include <signal.h>

#define BT_BUF_SIZE 100

/* If you want to suppress the messages in install_segv_handler, declare the
 * following variable as extern and set it to 0. */
static int ___install_segv_handler_verbose = 1;

static void
___segv_handler(int sig)
{
    if (___install_segv_handler_verbose != 1)
        return;

    void *buffer[LKT_BACKLOG];
    int nptrs = backtrace(buffer, LKT_BACKLOG);
    LOG_ERROR("SEGV", "Got signal %d", sig);

    char **strings = backtrace_symbols(buffer, nptrs);
    if (strings == NULL) {
        LOG_ERROR("SEGV", "No backtrace symbol found, abort");
        exit(EXIT_FAILURE);
    }

    for (int j = 0; j < nptrs; j++)
        LOG_ERROR("SEGV", "[%2d] %s", j, strings[j]);

    free(strings);
    exit(EXIT_FAILURE);
}

void
lkt_segv_quiet(void)
{
    ___install_segv_handler_verbose = 0;
}

void
lkt_segv_verbose(void)
{
    ___install_segv_handler_verbose = 1;
}

void
lkt_install_segv_handler(void)
{
    if (SIG_ERR == signal(SIGSEGV, ___segv_handler) && ___install_segv_handler_verbose == 1)
        LOG_WARN("SIGNAL", "Failed to install the segv handler");
    if (___install_segv_handler_verbose == 1)
        LOG_INFO("SIGNAL", "The segv handler has been installed");
}
