#if !defined(__LKT_THREAD_H__)
#define __LKT_THREAD_H__
#define _POSIX_C_SOURCE 200809L

#include <lektor/lktmodule.h>
#include <lektor/common.h>
#include <sys/types.h>

struct poller_thread {
    /* The input poll. */
    struct stack input;
    pthread_mutex_t input_lock;

    pthread_t th;
    unsigned int initial_size;

    /* The output pool. */
    struct stack output;
    pthread_mutex_t output_lock;
};

struct poller_thread_arg {
    struct poller_thread *self;
    void *args;
};

struct ___poller_thread_args {
    void *(*start)(struct poller_thread_arg *);
    struct poller_thread_arg *arg;
};

static void *
___poller_thread_start(void *args__)
{
    struct ___poller_thread_args *args         = (struct ___poller_thread_args *)args__;
    void *(*start)(struct poller_thread_arg *) = args->start;
    struct poller_thread_arg *arg              = args->arg;
    free(args__);
    return start(arg);
}

UNUSED static int
poller_new(struct poller_thread *th, void *(*func)(struct poller_thread_arg *), void *args)
{
    pthread_mutex_t mtx1     = PTHREAD_MUTEX_INITIALIZER;
    pthread_mutex_t mtx2     = PTHREAD_MUTEX_INITIALIZER;
    struct poller_thread th_ = {
        .input_lock  = mtx1,
        .output_lock = mtx2,
    };

    stack_new(&th_.input);
    stack_new(&th_.output);

    struct ___poller_thread_args *__args;
    *th               = th_;
    __args            = LKT_ALLOC_STRUCT(___poller_thread_args);
    __args->start     = func;
    __args->arg       = args;
    __args->arg->self = th;

    if (pthread_create(&(th->th), NULL, ___poller_thread_start, __args)) {
        LOG_ERROR("THREAD", "Failed to create a poller thread");
        return 1;
    }

    LOG_INFO("THREAD", "Create a new poller thread");
    return 0;
}

UNUSED static int
poller_join(struct poller_thread *th, void **ret)
{
    int sta = pthread_join(th->th, ret);

    if (sta)
        LOG_ERROR("THREAD", "Failed to join thread");

    stack_free(&th->input);
    stack_free(&th->output);
    LOG_WARN("THREAD", "Thread joined");

    memset(th, 0, sizeof(struct poller_thread));
    return sta;
}

UNUSED static int
th_append(struct stack *lst, pthread_mutex_t *lock, void *ptr)
{
    RETURN_IF(pthread_mutex_lock(lock), "Failed to lock", 1);
    stack_push(lst, ptr);
    RETURN_IF(pthread_mutex_unlock(lock), "Failed to lock", 1);
    return 0;
}

UNUSED static int
th_pop(struct stack *lst, pthread_mutex_t *lock, void **ptr)
{
    RETURN_IF(pthread_mutex_lock(lock), "Failed to lock", 1);
    int ret = stack_pop(lst, ptr);
    RETURN_IF(pthread_mutex_unlock(lock), "Failed to lock", 1);
    return ret;
}

UNUSED static int
th_head(struct stack *lst, pthread_mutex_t *lock, void **ptr)
{
    RETURN_IF(pthread_mutex_lock(lock), "Failed to lock", 1);
    int ret = stack_head(lst, ptr);
    RETURN_IF(pthread_mutex_unlock(lock), "Failed to lock", 1);
    return ret;
}

#define __POLLER_ATOMICS_DEFINE(operation, inout, level)                                  \
    UNUSED static int poller_##operation##inout(struct poller_thread *th, void level ptr) \
    {                                                                                     \
        return th_##operation(&th->inout, &th->inout##_lock, ptr);                        \
    }
// clang-format off
__POLLER_ATOMICS_DEFINE(append, input,  *);
__POLLER_ATOMICS_DEFINE(append, output, *);
__POLLER_ATOMICS_DEFINE(pop,    input,  **);
__POLLER_ATOMICS_DEFINE(pop,    output, **);
__POLLER_ATOMICS_DEFINE(head,   input,  **);
__POLLER_ATOMICS_DEFINE(head,   output, **);
// clang-format on
#undef __POLLER_ATOMICS_DEFINE

#endif /* __LKT_THREAD_H__ */
