#pragma once

#include <pthread.h>
#include <stdlib.h>

typedef void *(*worker_function)(void *, void *);

typedef enum {
    WORKER_STATUS_NONE      = 0,
    WORKER_STATUS_INTERRUPT = (1 << 1),
} WORKER_STATUS;

struct worker_pool {
    volatile size_t len;
    volatile size_t size;
    volatile size_t thread_size;
    volatile size_t thread_working;
    volatile worker_function *functions;
    volatile void *volatile *volatile args;
    pthread_t *threads;
    pthread_mutex_t lock;
    volatile int exit;
};

/* After a free, the pool is no longer usable, you must re-init with a new */
int worker_pool_new(struct worker_pool *, size_t init_len, size_t thread_count);
void worker_pool_free(struct worker_pool *);
void worker_pool_waitall(struct worker_pool *);

/* Request an interupt for all threads/tasks, you can wait after that. Note
 * that if a task is currently executed, it needs to supports a way of being
 * interupted (no automagic stuff here). */
void worker_pool_interrupt(struct worker_pool *);

/* It is up to the user to free or not the `arg` depending on what he whats */
int worker_pool_push(struct worker_pool *, worker_function, void *arg);

/* Get the working thread count */
size_t worker_pool_get_working_count(struct worker_pool *);

/* For clients, get the status of the worker */
WORKER_STATUS worker_pool_get_status(void *);
