#define _POSIX_C_SOURCE 200809L

#include <lektor/common.h>
#include <lektor/net.h>

PRIVATE_FUNCTION void
___crop_string(char *string, const size_t max_len)
{
    if (strlen(string) > max_len) {
        string[max_len] = '\0';
    }
}

struct lkt_command
lkt_command_parse(char *raw)
{
    errno = 0;
    char *endptr;
    bool skip_continuation = false;
    struct lkt_command res = { .name = raw, .args = { 0 }, .cont = strtol(raw, &endptr, 10) };

    /* Do we need the continuation? */
    if (errno != 0) {
        res.cont          = 0;
        skip_continuation = true;
    } else if (endptr == raw) {
        res.cont          = 0;
        skip_continuation = true;
    }

    /* Need the continuation here. */
    if (!skip_continuation) {
        /* That thing is an error. */
        if (*endptr == '\0') {
            ___crop_string(res.name, LKT_COMMAND_LEN_MAX);
            return res;
        }

        /* Get the real command name and not the continuation number. */
        raw += strspn(raw, " 1234567890");
        if (!*raw) {
            /* No real name found, only got integers, we will fail later. */
            ___crop_string(res.name, LKT_COMMAND_LEN_MAX);
            return res;
        }
        res.name = raw;
    }

    /* Get the args. We skip the first word in the 'raw'. */
    raw += strcspn(raw, " ");

    /* We also restrict the number of characters of the command, to avoid some
     * incredibly hight recursive calls... */
    ___crop_string(res.name, LKT_COMMAND_LEN_MAX);

    if (!*raw) {
        /* Here we have only one argument. */
        return res;
    }

    *raw++ = 0;
    raw += strspn(raw, " ");

    for (size_t i = 0; i < LKT_MESSAGE_ARGS_MAX && *raw; i++) {
        if (*raw == '"') {
            res.args[i] = ++raw;
            raw += strcspn(raw, "\"");
            if (*raw) {
                *raw++ = 0;
                raw += strcspn(raw, " ");
                *raw++ = 0;
            }
        }

        else if (*raw == '\'') {
            res.args[i] = ++raw;
            raw += strcspn(raw, "'");
            if (*raw) {
                *raw++ = 0;
                raw += strcspn(raw, " ");
                *raw++ = 0;
            }
        }

        else {
            res.args[i] = raw;
            raw += strcspn(raw, " ");
            *raw++ = 0;
        }

        raw += strspn(raw, " ");
    }

    /* The command has arguments. */
    return res;
}

struct lkt_command
lkt_command_clone(struct lkt_command *c)
{
    struct lkt_command res = {
        .name = strdup(c->name),
        .args = { 0 },
    };

    for (size_t i = 0; c->args[i] && i < LKT_MESSAGE_ARGS_MAX; i++)
        res.args[i] = strdup(c->args[i]);

    return res;
}

void
lkt_command_free(struct lkt_command *c)
{
    free(c->name);
    for (size_t i = 0; c->args[i]; i++)
        free(c->args[i]);
}
